/****************************************************************+
 *
 *  Conversion from Eulumdat to IESNA - standard
 *
 *  Copyright (c) 2001 Carsten Bauer (Berlin)
 *
 **************************************************************** */


/***************************************************************************

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation. This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
details.

*************************************************************************+ */


#include <string.h>
#include <stdio.h>
#include <stdlib.h>


/*
 ***********************file handling *************+ */
char inputfile[128];
char outputfile[128];

FILE* inputstream;
FILE* convinputstream;
FILE* outputstream;
FILE* lightdatastr;

int convert=0;

/*
 ******************conversion arrays  *******************+ */

char*** A;              /* string Array holding IESNA-output */
char** E;               /* string Array holding Eulumdat Input */
char*** E2;              /* add. Eul. data for each lamp type   */

int C[33][2] = {            /* conversion  table */
/* 0  */
   0,0,

/* 1     2     3     4      5      6      7      8       9      10  */
   0,0,  0,0,  0,0,  1,5,   0,0,   1,4,   0,0,   0,0,    0,0,   0,0,
   
   0,0,  0,0,  0,0,  0,0,   0,0,   1,9,   1,8,   1,10,   0,0,   0,0,

   0,0,  0,0,  0,0,  0,0,   0,0,   0,0,   1,1,   0,0,    1,2,   0,0,
   
   0,0,  2,3
};




/*
 ******************* intermediate variables *************** */

int Ng;
int Mc, Mc1, Mc2;              
int pos26, pos28, pos29, pos30;
int Isym;
int lumen; 
int pow;
int numlamp;
double test;
int numhzangs, numvtangs;

char*** cdvalbuf;


/*
 ******************+ functions  **************************+ */

void msunconv(char*);
void readeufile(FILE*);
void writeiefile(char*);
void converr(char*); 


#define setAC(x) strcpy(A[C[x][0]][C[x][1]],E[x])
#define setAC2(x,y) strcpy(A[C[x][0]][C[x][1]],y)



int
main(int argc, char** argv)
{
int i, j, k, cnt, ncnt;
int r, s, t;
char cbuf[256];
int buf;
int byte;
float flbufbr, flbufle;
char output[128];

switch(argc){

 case 3:
    if(strcmp("-c",argv[1]) == 0){
    convert = 1;
    strcpy(inputfile,argv[2]);
    msunconv(inputfile);
    }
    else 
      converr("bad option");
    break;
 
 case 2:
    if(strcmp("-c",argv[1]) == 0)
             converr("missing input file");
    else
      strcpy(inputfile,argv[1]);
    break;  
      
 case 1:
    converr("Usage: eumelconv [-c] filename \n");
    break;
  
  default:
    converr("Illegal options");  
}

                    

/* allocate arrays */

E = (char**)calloc(100, sizeof(char*));
  for(i=0; i<100; i++) E[i] = (char*)malloc(80*sizeof(char));


A = (char***)calloc(6, sizeof(char**));
     for(i=0; i<6; i++){
        A[i] = (char**)calloc(11, sizeof(char*));
         for(j=0; j<11; j++) A[i][j]=(char*)malloc(10*sizeof(char));
     }   

if(convert){
     rewind(convinputstream);
     readeufile(convinputstream);
     fclose(convinputstream);
     
}
else{
     if((inputstream=fopen(inputfile, "r"))==NULL)
              converr("opening of eulumdat file failed");
     readeufile(inputstream);
     fclose(inputstream);
}              
                        

pos28=26+10+1;         /* determine positions for angles */
pos29=pos28+Mc;
pos30=pos29+Ng;

/************************************* angles ********************/

Isym = atoi(E[3]);   
                             /* check symmetry */
if(Isym == 3) converr("symmetry type 3 not convertable in this version");

switch(Isym){             /* determine reduced num of horiz angles */

  case 0:
   numhzangs = Mc;
  break; 

  case 1:
   numhzangs = 1;
  break;
  
  case 2:
   Mc2 = Mc/2 +1;
   numhzangs = Mc2;
  break;

  case 4:
   Mc2 = Mc/4 +1;
   numhzangs = Mc2;
}


                         /*  allocate candelavalue-array */
cdvalbuf = (char***)calloc(numhzangs, sizeof(char**));
for(r=0; r<numhzangs; r++){
         cdvalbuf[r] = (char**)calloc(Ng, sizeof(char*));
         for(s=0; s<Ng; s++)
                cdvalbuf[r][s] = (char*)malloc(8*sizeof(char));
}                                          

if(Isym == 0)
   sprintf( cbuf, "%i", numhzangs+1);
else
   sprintf( cbuf, "%i", numhzangs);
setAC2(4,cbuf);        /* set angles for output */
setAC(6);            

/********************************** dimensions of light emitting surface */
                        /* convert from mm to feet on the way out ... */
flbufbr = atof(E[17]);
flbufle = atof(E[16]);

if (flbufbr < 0.000001){
    sprintf(cbuf, "%7.3f", -flbufle*3.2808/1000.0);
    setAC2(17,cbuf);
    sprintf(cbuf, "0.0");
    setAC2(16,cbuf);
}
else {
   sprintf(cbuf, "%7.3f", flbufbr*3.2808/1000.0);
   setAC2(17,cbuf);
   sprintf(cbuf, "%7.3f", flbufle*3.2808/1000.0);
   setAC2(16,cbuf);
}
 
flbufbr= atof(E[18]) + atof(E[19]) + atof(E[20]) + atof(E[21]);
sprintf(cbuf, "%7.3f", flbufbr*3.2808/4000);
setAC2(18,cbuf);


             /* build outputfilename  */
strncpy(outputfile, inputfile, (strlen(inputfile)-4));             


/*********************************** write iesna-file for each subset */
for(i=1; i<=pos26; i++){
  if(pos26>1) sprintf(output, "%s_%i.ies", outputfile, i);
    else sprintf(output, "%s.ies",outputfile);

 setAC2(27,E2[i][1]);                /* nr of lamps, lumens, power cons. */
 numlamp=atoi(E2[i][1]);
 lumen=atoi(E2[i][3]);
 sprintf(cbuf, "%8.2f", (float)lumen/(float)numlamp);
 setAC2(29,cbuf);
 setAC2(32,E2[i][6]);



                               /* "default" rest of A values */
 A[1][3] = "1.0"; A[1][6]="1"; A[1][7]="1"; A[2][1]="1"; A[2][2]="1";

 outputstream=fopen(output, "w");
 fputs("IESNA93\n" ,outputstream);
 sprintf(cbuf, "converted by eulum2ies from: %s\n", E[11]);
 fputs(cbuf, outputstream);
 fputs("TILT=NONE\n" , outputstream);


 for(k=1; k<= 10; k++){                 /* line 1 */
   sprintf(cbuf, "%s ", A[1][k]);
   fputs(cbuf, outputstream);
 }
 fputs("\n", outputstream);

 for(k=1; k<=3; k++){                   /* line 2 */
  sprintf(cbuf, "%s ", A[2][k]);
  fputs(cbuf, outputstream); 
 }
 fputs("\n", outputstream);

 cnt=0;
 for(k=pos29; k<pos29+Ng; k++){         /* line 3  (vert. angles ) */
   sprintf(cbuf, "%s ", E[k]);
   fputs(cbuf, outputstream);
   if(++cnt == 24){cnt =0; fputs("\n", outputstream);}
 }
 fputs("\n", outputstream);

 cnt=0;                                /* line 4 (horiz. angles) */
 if(Isym==2 || Isym==0){  
  /* 2fold sym needs special treatment (-> for simplicity no sym in iesna) */ 
  for(k=pos28; k<pos28+Mc; k++){  
    sprintf(cbuf, "%s ", E[k]);
    fputs(cbuf, outputstream);
    if(++cnt == 25){cnt =0; fputs("\n", outputstream);}
  }
  fputs("360.00\n",outputstream);  /* 0 -> 360 repeated */
  cnt = 0;
 } 
 else{ 
  for(k=pos28; k<pos28+numhzangs; k++){  
    sprintf(cbuf, "%s ", E[k]);
    fputs(cbuf, outputstream);
    if(++cnt == 24){cnt =0; fputs("\n", outputstream);}
  }
 }
 if(cnt)
    fputs("\n", outputstream);

 cnt=0;
 ncnt=0;
 rewind(lightdatastr);

  /* read in cd/klm vals, convert to cd , buffer vals */

for(r=0; r<numhzangs; r++){
  for(s=0; s<Ng; s++){
    fgets(cbuf,80, lightdatastr);
    *(cbuf + strlen(cbuf)-1)='\0';
    sprintf(cdvalbuf[r][s], "%8.1f ", (atof(cbuf)*(float)lumen/1000));
  }
}    

/* write them out.. a bit complicated because of different orientation
  of C-planes in both standards ... */

switch(Isym){

case 1:
for(r=0; r<numhzangs; r++){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 } 
}
break;

case 4:
/* reversal of angs is sufficient because of 4-fold-symmetry */
for(r=numhzangs-1; r>=0; r--){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
}
break;

case 0:
/* vals from 270 deg to 360 deg mapped to 0 to 90 */
for(r=(numhzangs-numhzangs/4); r<numhzangs; r++){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
}
/* the rest */
for(r=0; r<(numhzangs-numhzangs/4); r++){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
}
/* repeat first val (hz=0) at hz = (360) */
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[(numhzangs-numhzangs/4)][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
break;

case 2:
  /* sorry for missing comment ... :) */
for(r=Mc/4; r>0; r--){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
}
for(r=0; r<Mc/4; r++){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
}
for(r=Mc/4; r<Mc/2; r++){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
}
for(r=Mc/2; r>=Mc/4; r--){
 for(s=0; s<Ng; s++){
   fputs(cdvalbuf[r][s], outputstream);
   cnt++; ncnt++;
   if(cnt == 10){cnt=0; fputs("\n", outputstream);}
   if(ncnt == Ng){cnt=0; ncnt =0; fputs("\n", outputstream);}
 }
}
break;

}
fclose(outputstream); 


}  /* next pos26 value */

fclose(lightdatastr);        
return(0);
}

void
readeufile(FILE* fstream)
{
int i, j, k, slen;
int rest;
int byte;

for(i=1;i<=26;i++){
    fgets(E[i], 80, fstream);
    *(E[i] + strlen(E[i])-1)='\0';
}

pos26= atoi(E[26]);            /* evaluate number of subsets */
Mc= atoi(E[4]);                /* set number of angles */
Ng= atoi(E[6]);


           /* allocate additional array E2 */
   E2 = (char***)calloc(pos26+1, sizeof(char**));
        for(j=0; j<=pos26; j++){
            E2[j] = (char**)calloc(7, sizeof(char*));
            for(k=0; k<=6; k++) E2[j][k]=(char*)malloc(24*sizeof(char));
        }    
                           /* read in E2 */       
   for(j=1; j<=pos26; j++)
           for(k=1;k<=6;k++){
               fgets(E2[j][k], 80, fstream);
               *(E2[j][k] + strlen(E2[j][k])-1)='\0';
           /* debug
               printf("E[%i][%i] = %s\n", j,k,E2[j][k]);
           */     
           }    

rest=10+Mc+Ng;         /* read in the rest of the main array */

for(i=27; i<=26+rest; i++){
          fgets(E[i], 80, fstream);
          *(E[i] + strlen(E[i])-1)='\0';
          /* debug
          printf("E[%i] = %s\n", i,E[i]);
          */
          }
                     
                      /* store the candela values in tmpfile */
lightdatastr = tmpfile();
while((byte=fgetc(fstream)) != EOF)
        fputc(byte, lightdatastr);

return;
}

void
msunconv(char* fname)
{

char buffer[80];
if((inputstream=fopen(fname, "r"))==NULL)
         converr("opening of eulumdat file failed");
convinputstream=tmpfile();
while(fgets(buffer, 80,inputstream) != NULL){
     *(buffer+strlen(buffer)-2)='\n' ;
     *(buffer+strlen(buffer)-1)='\0' ;
     
     fputs(buffer,convinputstream);
}     

}         

void
converr(char* msg)
{
printf("Error: %s\n", msg);
exit(1);
}


















 









